package com.sun.corba.se.impl.protocol;

import com.sun.corba.se.impl.orbutil.ORBUtility;
import com.sun.corba.se.impl.encoding.CodeSetComponentInfo;
import com.sun.corba.se.spi.servicecontext.CodeSetServiceContext;
import java.util.Iterator;
import com.sun.corba.se.spi.ior.iiop.CodeSetsComponent;
import com.sun.corba.se.impl.encoding.CodeSetConversion;
import com.sun.corba.se.spi.ior.iiop.IIOPProfileTemplate;
import java.io.IOException;
import com.sun.corba.se.pept.broker.Broker;
import com.sun.corba.se.impl.encoding.EncapsInputStream;
import sun.corba.EncapsInputStreamFactory;
import com.sun.corba.se.spi.servicecontext.UnknownServiceContext;
import com.sun.corba.se.spi.ior.IOR;
import com.sun.corba.se.spi.servicecontext.ServiceContext;
import com.sun.corba.se.spi.ior.iiop.GIOPVersion;
import com.sun.corba.se.spi.servicecontext.SendingContextServiceContext;
import com.sun.corba.se.spi.servicecontext.ORBVersionServiceContext;
import com.sun.corba.se.spi.orb.ORBVersionFactory;
import com.sun.corba.se.spi.servicecontext.MaxStreamFormatVersionServiceContext;
import com.sun.corba.se.pept.protocol.MessageMediator;
import com.sun.corba.se.spi.transport.CorbaContactInfoListIterator;
import com.sun.corba.se.spi.servicecontext.ServiceContexts;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.portable.UnknownException;
import com.sun.corba.se.spi.servicecontext.UEInfoServiceContext;
import org.omg.CORBA_2_3.portable.InputStream;
import com.sun.corba.se.impl.encoding.CDRInputObject;
import org.omg.CORBA.portable.ApplicationException;
import com.sun.corba.se.pept.encoding.InputObject;
import com.sun.corba.se.pept.transport.OutboundConnectionCache;
import org.omg.CORBA.portable.RemarshalException;
import com.sun.corba.se.impl.logging.ORBUtilSystemException;
import com.sun.corba.se.spi.protocol.CorbaMessageMediator;
import com.sun.corba.se.spi.transport.CorbaConnection;
import com.sun.corba.se.spi.orb.ORB;
import com.sun.corba.se.spi.transport.CorbaContactInfo;
import com.sun.corba.se.pept.encoding.OutputObject;
import java.util.concurrent.ConcurrentHashMap;
import com.sun.corba.se.pept.transport.ContactInfo;
import java.util.concurrent.ConcurrentMap;
import com.sun.corba.se.pept.protocol.ClientRequestDispatcher;

public class CorbaClientRequestDispatcherImpl implements ClientRequestDispatcher
{
    private ConcurrentMap<ContactInfo, Object> locks;
    
    public CorbaClientRequestDispatcherImpl() {
        this.locks = new ConcurrentHashMap();
    }
    
    public OutputObject beginRequest(Object o, String str, boolean b, ContactInfo obj) {
        ORB orb = null;
        try {
            CorbaContactInfo corbaContactInfo = (CorbaContactInfo)obj;
            orb = (ORB)obj.getBroker();
            if (orb.subcontractDebugFlag) {
                this.dprint(".beginRequest->: op/" + str);
            }
            orb.getPIHandler().initiateClientPIRequest(false);
            CorbaConnection corbaConnection = null;
            Object o2 = this.locks.get(obj);
            if (o2 == null) {
                Object o3 = new Object();
                o2 = this.locks.putIfAbsent(obj, o3);
                if (o2 == null) {
                    o2 = o3;
                }
            }
            synchronized (o2) {
                if (obj.isConnectionBased()) {
                    if (obj.shouldCacheConnection()) {
                        corbaConnection = (CorbaConnection)orb.getTransportManager().getOutboundConnectionCache(obj).get(obj);
                    }
                    if (corbaConnection != null) {
                        if (orb.subcontractDebugFlag) {
                            this.dprint(".beginRequest: op/" + str + ": Using cached connection: " + corbaConnection);
                        }
                    }
                    else {
                        try {
                            corbaConnection = (CorbaConnection)obj.createConnection();
                            if (orb.subcontractDebugFlag) {
                                this.dprint(".beginRequest: op/" + str + ": Using created connection: " + corbaConnection);
                            }
                        }
                        catch (RuntimeException obj2) {
                            if (orb.subcontractDebugFlag) {
                                this.dprint(".beginRequest: op/" + str + ": failed to create connection: " + obj2);
                            }
                            if (this.getContactInfoListIterator(orb).reportException(obj, obj2)) {
                                if (this.getContactInfoListIterator(orb).hasNext()) {
                                    obj = (ContactInfo)this.getContactInfoListIterator(orb).next();
                                    this.unregisterWaiter(orb);
                                    OutputObject beginRequest = this.beginRequest(o, str, b, obj);
                                }
                                else {
                                    throw obj2;
                                }
                            }
                            else {
                                throw obj2;
                            }
                        }
                        if (corbaConnection.shouldRegisterReadEvent()) {
                            orb.getTransportManager().getSelector(0).registerForEvent(corbaConnection.getEventHandler());
                            corbaConnection.setState("ESTABLISHED");
                        }
                        if (obj.shouldCacheConnection()) {
                            OutboundConnectionCache outboundConnectionCache = orb.getTransportManager().getOutboundConnectionCache(obj);
                            outboundConnectionCache.stampTime(corbaConnection);
                            outboundConnectionCache.put(obj, corbaConnection);
                        }
                    }
                }
            }
            CorbaMessageMediator clientPIInfo = (CorbaMessageMediator)obj.createMessageMediator(orb, obj, corbaConnection, str, b);
            if (orb.subcontractDebugFlag) {
                this.dprint(".beginRequest: " + this.opAndId(clientPIInfo) + ": created message mediator: " + clientPIInfo);
            }
            orb.getInvocationInfo().setMessageMediator(clientPIInfo);
            if (corbaConnection != null && corbaConnection.getCodeSetContext() == null) {
                this.performCodeSetNegotiation(clientPIInfo);
            }
            this.addServiceContexts(clientPIInfo);
            OutputObject outputObject = obj.createOutputObject(clientPIInfo);
            if (orb.subcontractDebugFlag) {
                this.dprint(".beginRequest: " + this.opAndId(clientPIInfo) + ": created output object: " + outputObject);
            }
            this.registerWaiter(clientPIInfo);
            synchronized (o2) {
                if (obj.isConnectionBased() && obj.shouldCacheConnection()) {
                    orb.getTransportManager().getOutboundConnectionCache(obj).reclaim();
                }
            }
            orb.getPIHandler().setClientPIInfo(clientPIInfo);
            try {
                orb.getPIHandler().invokeClientPIStartingPoint();
            }
            catch (RemarshalException ex) {
                if (orb.subcontractDebugFlag) {
                    this.dprint(".beginRequest: " + this.opAndId(clientPIInfo) + ": Remarshal");
                }
                if (this.getContactInfoListIterator(orb).hasNext()) {
                    obj = (ContactInfo)this.getContactInfoListIterator(orb).next();
                    if (orb.subcontractDebugFlag) {
                        this.dprint("RemarshalException: hasNext true\ncontact info " + obj);
                    }
                    orb.getPIHandler().makeCompletedClientRequest(3, null);
                    this.unregisterWaiter(orb);
                    orb.getPIHandler().cleanupClientPIRequest();
                    OutputObject beginRequest2 = this.beginRequest(o, str, b, obj);
                }
                else {
                    if (orb.subcontractDebugFlag) {
                        this.dprint("RemarshalException: hasNext false");
                    }
                    throw ORBUtilSystemException.get(orb, "rpc.protocol").remarshalWithNowhereToGo();
                }
            }
            clientPIInfo.initializeMessage();
            if (orb.subcontractDebugFlag) {
                this.dprint(".beginRequest: " + this.opAndId(clientPIInfo) + ": initialized message");
            }
        }
        finally {
            if (orb.subcontractDebugFlag) {
                this.dprint(".beginRequest<-: op/" + str);
            }
        }
    }
    
    public InputObject marshalingComplete(Object o, OutputObject outputObject) throws ApplicationException, RemarshalException {
        ORB orb = null;
        CorbaMessageMediator corbaMessageMediator = null;
        try {
            corbaMessageMediator = (CorbaMessageMediator)outputObject.getMessageMediator();
            orb = (ORB)corbaMessageMediator.getBroker();
            if (orb.subcontractDebugFlag) {
                this.dprint(".marshalingComplete->: " + this.opAndId(corbaMessageMediator));
            }
            InputObject processResponse = this.processResponse(orb, corbaMessageMediator, this.marshalingComplete1(orb, corbaMessageMediator));
        }
        finally {
            if (orb.subcontractDebugFlag) {
                this.dprint(".marshalingComplete<-: " + this.opAndId(corbaMessageMediator));
            }
        }
    }
    
    public InputObject marshalingComplete1(ORB orb, CorbaMessageMediator corbaMessageMediator) throws ApplicationException, RemarshalException {
        try {
            corbaMessageMediator.finishSendingRequest();
            if (orb.subcontractDebugFlag) {
                this.dprint(".marshalingComplete: " + this.opAndId(corbaMessageMediator) + ": finished sending request");
            }
            return corbaMessageMediator.waitForResponse();
        }
        catch (RuntimeException ex) {
            if (orb.subcontractDebugFlag) {
                this.dprint(".marshalingComplete: " + this.opAndId(corbaMessageMediator) + ": exception: " + ex.toString());
            }
            int reportException = this.getContactInfoListIterator(orb).reportException(corbaMessageMediator.getContactInfo(), ex);
            Exception invokeClientPIEndingPoint = orb.getPIHandler().invokeClientPIEndingPoint(2, ex);
            if (!(reportException == 0)) {
                if (invokeClientPIEndingPoint == ex) {
                    this.continueOrThrowSystemOrRemarshal(corbaMessageMediator, new RemarshalException());
                }
                else {
                    this.continueOrThrowSystemOrRemarshal(corbaMessageMediator, invokeClientPIEndingPoint);
                }
                return null;
            }
            else {
                if ((invokeClientPIEndingPoint instanceof RuntimeException)) {
                    throw (RuntimeException)invokeClientPIEndingPoint;
                }
                if ((invokeClientPIEndingPoint instanceof RemarshalException)) {
                    throw (RemarshalException)invokeClientPIEndingPoint;
                }
                throw ex;
            }
        }
    }
    
    protected InputObject processResponse(ORB orb, CorbaMessageMediator corbaMessageMediator, InputObject inputObject) throws ApplicationException, RemarshalException {
        ORBUtilSystemException value = ORBUtilSystemException.get(orb, "rpc.protocol");
        if (orb.subcontractDebugFlag) {
            this.dprint(".processResponse: " + this.opAndId(corbaMessageMediator) + ": response received");
        }
        if (corbaMessageMediator.getConnection() != null) {
            ((CorbaConnection)corbaMessageMediator.getConnection()).setPostInitialContexts();
        }
        Exception ex = null;
        if (corbaMessageMediator.isOneWay()) {
            this.getContactInfoListIterator(orb).reportSuccess(corbaMessageMediator.getContactInfo());
            this.continueOrThrowSystemOrRemarshal(corbaMessageMediator, orb.getPIHandler().invokeClientPIEndingPoint(0, ex));
            return null;
        }
        this.consumeServiceContexts(orb, corbaMessageMediator);
        ((CDRInputObject)inputObject).performORBVersionSpecificInit();
        if (corbaMessageMediator.isSystemExceptionReply()) {
            SystemException systemExceptionReply = corbaMessageMediator.getSystemExceptionReply();
            if (orb.subcontractDebugFlag) {
                this.dprint(".processResponse: " + this.opAndId(corbaMessageMediator) + ": received system exception: " + systemExceptionReply);
            }
            if (this.getContactInfoListIterator(orb).reportException(corbaMessageMediator.getContactInfo(), systemExceptionReply)) {
                Exception invokeClientPIEndingPoint3 = orb.getPIHandler().invokeClientPIEndingPoint(2, systemExceptionReply);
                if (systemExceptionReply == invokeClientPIEndingPoint3) {
                    this.continueOrThrowSystemOrRemarshal(corbaMessageMediator, new RemarshalException());
                    throw value.statementNotReachable1();
                }
                this.continueOrThrowSystemOrRemarshal(corbaMessageMediator, invokeClientPIEndingPoint3);
                throw value.statementNotReachable2();
            }
            else {
                ServiceContexts replyServiceContexts = corbaMessageMediator.getReplyServiceContexts();
                if (replyServiceContexts != null) {
                    UEInfoServiceContext ueInfoServiceContext = (UEInfoServiceContext)replyServiceContexts.get(9);
                    if (ueInfoServiceContext != null) {
                        this.continueOrThrowSystemOrRemarshal(corbaMessageMediator, orb.getPIHandler().invokeClientPIEndingPoint(2, new UnknownException(ueInfoServiceContext.getUE())));
                        throw value.statementNotReachable3();
                    }
                }
                this.continueOrThrowSystemOrRemarshal(corbaMessageMediator, orb.getPIHandler().invokeClientPIEndingPoint(2, systemExceptionReply));
                throw value.statementNotReachable4();
            }
        }
        else if (corbaMessageMediator.isUserExceptionReply()) {
            if (orb.subcontractDebugFlag) {
                this.dprint(".processResponse: " + this.opAndId(corbaMessageMediator) + ": received user exception");
            }
            this.getContactInfoListIterator(orb).reportSuccess(corbaMessageMediator.getContactInfo());
            String peekUserExceptionId = this.peekUserExceptionId(inputObject);
            Exception unmarshalDIIUserException;
            Exception diiException;
            if (corbaMessageMediator.isDIIRequest()) {
                unmarshalDIIUserException = corbaMessageMediator.unmarshalDIIUserException(peekUserExceptionId, (InputStream)inputObject);
                diiException = orb.getPIHandler().invokeClientPIEndingPoint(1, unmarshalDIIUserException);
                corbaMessageMediator.setDIIException(diiException);
            }
            else {
                diiException = orb.getPIHandler().invokeClientPIEndingPoint(1, unmarshalDIIUserException = new ApplicationException(peekUserExceptionId, (org.omg.CORBA.portable.InputStream)inputObject));
            }
            if (diiException != unmarshalDIIUserException) {
                this.continueOrThrowSystemOrRemarshal(corbaMessageMediator, diiException);
            }
            if ((diiException instanceof ApplicationException)) {
                throw (ApplicationException)diiException;
            }
            return inputObject;
        }
        else if (corbaMessageMediator.isLocationForwardReply()) {
            if (orb.subcontractDebugFlag) {
                this.dprint(".processResponse: " + this.opAndId(corbaMessageMediator) + ": received location forward");
            }
            this.getContactInfoListIterator(orb).reportRedirect((CorbaContactInfo)corbaMessageMediator.getContactInfo(), corbaMessageMediator.getForwardedIOR());
            Exception invokeClientPIEndingPoint2 = orb.getPIHandler().invokeClientPIEndingPoint(3, null);
            if (!(invokeClientPIEndingPoint2 instanceof RemarshalException)) {
                ex = invokeClientPIEndingPoint2;
            }
            if (ex != null) {
                this.continueOrThrowSystemOrRemarshal(corbaMessageMediator, ex);
            }
            this.continueOrThrowSystemOrRemarshal(corbaMessageMediator, new RemarshalException());
            throw value.statementNotReachable5();
        }
        else if (corbaMessageMediator.isDifferentAddrDispositionRequestedReply()) {
            if (orb.subcontractDebugFlag) {
                this.dprint(".processResponse: " + this.opAndId(corbaMessageMediator) + ": received different addressing dispostion request");
            }
            this.getContactInfoListIterator(orb).reportAddrDispositionRetry((CorbaContactInfo)corbaMessageMediator.getContactInfo(), corbaMessageMediator.getAddrDispositionReply());
            Exception invokeClientPIEndingPoint = orb.getPIHandler().invokeClientPIEndingPoint(5, null);
            if (!(invokeClientPIEndingPoint instanceof RemarshalException)) {
                ex = invokeClientPIEndingPoint;
            }
            if (ex != null) {
                this.continueOrThrowSystemOrRemarshal(corbaMessageMediator, ex);
            }
            this.continueOrThrowSystemOrRemarshal(corbaMessageMediator, new RemarshalException());
            throw value.statementNotReachable6();
        }
        else {
            if (orb.subcontractDebugFlag) {
                this.dprint(".processResponse: " + this.opAndId(corbaMessageMediator) + ": received normal response");
            }
            this.getContactInfoListIterator(orb).reportSuccess(corbaMessageMediator.getContactInfo());
            corbaMessageMediator.handleDIIReply((InputStream)inputObject);
            this.continueOrThrowSystemOrRemarshal(corbaMessageMediator, orb.getPIHandler().invokeClientPIEndingPoint(0, null));
            return inputObject;
        }
    }
    
    protected void continueOrThrowSystemOrRemarshal(CorbaMessageMediator corbaMessageMediator, Exception obj) throws SystemException, RemarshalException {
        ORB orb = (ORB)corbaMessageMediator.getBroker();
        if (obj == null) {
            return;
        }
        if ((obj instanceof RemarshalException)) {
            orb.getInvocationInfo().setIsRetryInvocation(true);
            this.unregisterWaiter(orb);
            if (orb.subcontractDebugFlag) {
                this.dprint(".continueOrThrowSystemOrRemarshal: " + this.opAndId(corbaMessageMediator) + ": throwing Remarshal");
            }
            throw (RemarshalException)obj;
        }
        else {
            if (orb.subcontractDebugFlag) {
                this.dprint(".continueOrThrowSystemOrRemarshal: " + this.opAndId(corbaMessageMediator) + ": throwing sex:" + obj);
            }
            throw (SystemException)obj;
        }
    }
    
    protected CorbaContactInfoListIterator getContactInfoListIterator(ORB orb) {
        return (CorbaContactInfoListIterator)((CorbaInvocationInfo)orb.getInvocationInfo()).getContactInfoListIterator();
    }
    
    protected void registerWaiter(CorbaMessageMediator corbaMessageMediator) {
        if (corbaMessageMediator.getConnection() != null) {
            corbaMessageMediator.getConnection().registerWaiter(corbaMessageMediator);
        }
    }
    
    protected void unregisterWaiter(ORB orb) {
        MessageMediator messageMediator = orb.getInvocationInfo().getMessageMediator();
        if (messageMediator != null && messageMediator.getConnection() != null) {
            messageMediator.getConnection().unregisterWaiter(messageMediator);
        }
    }
    
    protected void addServiceContexts(CorbaMessageMediator corbaMessageMediator) {
        ORB orb = (ORB)corbaMessageMediator.getBroker();
        CorbaConnection corbaConnection = (CorbaConnection)corbaMessageMediator.getConnection();
        GIOPVersion giopVersion = corbaMessageMediator.getGIOPVersion();
        ServiceContexts requestServiceContexts = corbaMessageMediator.getRequestServiceContexts();
        this.addCodeSetServiceContext(corbaConnection, requestServiceContexts, giopVersion);
        requestServiceContexts.put(MaxStreamFormatVersionServiceContext.singleton);
        requestServiceContexts.put(new ORBVersionServiceContext(ORBVersionFactory.getORBVersion()));
        if (corbaConnection != null && !corbaConnection.isPostInitialContexts()) {
            requestServiceContexts.put(new SendingContextServiceContext(orb.getFVDCodeBaseIOR()));
        }
    }
    
    protected void consumeServiceContexts(ORB orb, CorbaMessageMediator corbaMessageMediator) {
        ServiceContexts replyServiceContexts = corbaMessageMediator.getReplyServiceContexts();
        ORBUtilSystemException value = ORBUtilSystemException.get(orb, "rpc.protocol");
        if (replyServiceContexts == null) {
            return;
        }
        ServiceContext value2 = replyServiceContexts.get(6);
        if (value2 != null) {
            IOR ior = ((SendingContextServiceContext)value2).getIOR();
            try {
                if (corbaMessageMediator.getConnection() != null) {
                    ((CorbaConnection)corbaMessageMediator.getConnection()).setCodeBaseIOR(ior);
                }
            }
            catch (ThreadDeath threadDeath) {
                throw threadDeath;
            }
            catch (Throwable t) {
                throw value.badStringifiedIor(t);
            }
        }
        ServiceContext value3 = replyServiceContexts.get(1313165056);
        if (value3 != null) {
            orb.setORBVersion(((ORBVersionServiceContext)value3).getVersion());
        }
        this.getExceptionDetailMessage(corbaMessageMediator, value);
        return;
    }
    
    protected void getExceptionDetailMessage(CorbaMessageMediator corbaMessageMediator, ORBUtilSystemException ex) {
        ServiceContext value = corbaMessageMediator.getReplyServiceContexts().get(14);
        if (value == null) {
            return;
        }
        if (!(value instanceof UnknownServiceContext)) {
            throw ex.badExceptionDetailMessageServiceContextType();
        }
        byte[] data = ((UnknownServiceContext)value).getData();
        EncapsInputStream encapsInputStream = EncapsInputStreamFactory.newEncapsInputStream((ORB)corbaMessageMediator.getBroker(), data, data.length);
        encapsInputStream.consumeEndian();
        corbaMessageMediator.setReplyExceptionDetailMessage("----------BEGIN server-side stack trace----------\n" + encapsInputStream.read_wstring() + "\n----------END server-side stack trace----------");
        return;
    }
    
    public void endRequest(Broker broker, Object o, InputObject inputObject) {
        ORB orb = (ORB)broker;
        try {
            if (orb.subcontractDebugFlag) {
                this.dprint(".endRequest->");
            }
            MessageMediator messageMediator = orb.getInvocationInfo().getMessageMediator();
            if (messageMediator != null) {
                if (messageMediator.getConnection() != null) {
                    ((CorbaMessageMediator)messageMediator).sendCancelRequestIfFinalFragmentNotSent();
                }
                InputObject inputObject2 = messageMediator.getInputObject();
                if (inputObject2 != null) {
                    inputObject2.close();
                }
                OutputObject outputObject = messageMediator.getOutputObject();
                if (outputObject != null) {
                    outputObject.close();
                }
            }
            this.unregisterWaiter(orb);
            orb.getPIHandler().cleanupClientPIRequest();
        }
        catch (IOException ex) {
            if (orb.subcontractDebugFlag) {
                this.dprint(".endRequest: ignoring IOException - " + ex.toString());
            }
        }
        finally {
            if (orb.subcontractDebugFlag) {
                this.dprint(".endRequest<-");
            }
        }
    }
    
    protected void performCodeSetNegotiation(CorbaMessageMediator corbaMessageMediator) {
        CorbaConnection corbaConnection = (CorbaConnection)corbaMessageMediator.getConnection();
        IOR effectiveTargetIOR = ((CorbaContactInfo)corbaMessageMediator.getContactInfo()).getEffectiveTargetIOR();
        GIOPVersion giopVersion = corbaMessageMediator.getGIOPVersion();
        if (corbaConnection != null && corbaConnection.getCodeSetContext() == null && !giopVersion.equals(GIOPVersion.V1_0)) {
            synchronized (corbaConnection) {
                if (corbaConnection.getCodeSetContext() != null) {
                    return;
                }
                Iterator<Object> iteratorById = ((IIOPProfileTemplate)effectiveTargetIOR.getProfile().getTaggedProfileTemplate()).iteratorById(1);
                if (!iteratorById.hasNext()) {
                    return;
                }
                corbaConnection.setCodeSetContext(CodeSetConversion.impl().negotiate(corbaConnection.getBroker().getORBData().getCodeSetComponentInfo(), ((CodeSetsComponent)iteratorById.next()).getCodeSetComponentInfo()));
            }
        }
    }
    
    protected void addCodeSetServiceContext(CorbaConnection corbaConnection, ServiceContexts serviceContexts, GIOPVersion giopVersion) {
        if (giopVersion.equals(GIOPVersion.V1_0) || corbaConnection == null) {
            return;
        }
        CodeSetComponentInfo.CodeSetContext codeSetContext = null;
        if (corbaConnection.getBroker().getORBData().alwaysSendCodeSetServiceContext() || !corbaConnection.isPostInitialContexts()) {
            codeSetContext = corbaConnection.getCodeSetContext();
        }
        if (codeSetContext == null) {
            return;
        }
        serviceContexts.put(new CodeSetServiceContext(codeSetContext));
        return;
    }
    
    protected String peekUserExceptionId(InputObject inputObject) {
        CDRInputObject cdrInputObject = (CDRInputObject)inputObject;
        cdrInputObject.mark(Integer.MAX_VALUE);
        String read_string = cdrInputObject.read_string();
        cdrInputObject.reset();
        return read_string;
    }
    
    protected void dprint(String s) {
        ORBUtility.dprint("CorbaClientRequestDispatcherImpl", s);
    }
    
    protected String opAndId(CorbaMessageMediator corbaMessageMediator) {
        return ORBUtility.operationNameAndRequestId(corbaMessageMediator);
    }
}
